/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "shopper.h"           // automake, i8n, gettext
#include <hildonmm.h>          // Hildon for C++
#include <iostream>            // IO Stream
#include <sstream>             // String Stream
#include <gconfmm.h>           // Gnome::Conf

#include "shopperList.h"       // Data objects
#include "ui/MainWindow.h"     // GUI

//#define DEBUG(...) cerr << __VA_ARGS__ 
#define DEBUG(...)

void cb_filename_changed(guint i, Gnome::Conf::Entry e)
{}

int main(int argc, char *argv[])
{
  // Initialize gtkmm:
  Gtk::Main kit(&argc, &argv);
  Hildon::init();

  setlocale ( LC_ALL, "" );
  bindtextdomain ( GETTEXT_PACKAGE, LOCALEDIR );
  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
  textdomain ( GETTEXT_PACKAGE );

  osso_context_t* osso_context = osso_initialize(PACKAGE_DBUS_NAME, PACKAGE_VERSION, TRUE /* deprecated parameter */, 0 /* Use default Glib main loop context */);
  if(!osso_context)
    std::cerr << "osso_initialize() failed." << std::endl;
  
  Glib::set_application_name("Shopper");
  Gnome::Conf::init();

  DEBUG("gconf get client\n");
  static Glib::RefPtr<Gnome::Conf::Client>
	  client = Gnome::Conf::Client::get_default_client();
	  

  Gnome::Conf::Entry filename_E;
  try {
	  DEBUG("gconf add dir\n");
	  std::auto_ptr< Glib::Error > err;
	  client->add_dir("/apps/maemo/shopper",
					  Gnome::Conf::CLIENT_PRELOAD_NONE, err);
	  if (err.get()){
		  DEBUG(err->what());
		  throw Gnome::Conf::Error(Gnome::Conf::Error::CORRUPT,err->what());
	  }
	  DEBUG("gconf get key\n");
	  filename_E = client->get_entry("/apps/maemo/shopper/list_filename");

  } catch(Gnome::Conf::Error err) {
	  DEBUG("gconf err\n");
	  exit (1);
  }

  DEBUG("gconf add notify\n");
  client->notify_add("/apps/maemo/shopper/list_filename",
					 &cb_filename_changed);
  
/* Load default list */
  Shopper::List* list;
  Glib::ustring filename = "";
  if (filename_E.get_value().get_type() == Gnome::Conf::VALUE_STRING) {
		DEBUG("gconf got key\n");
		filename = filename_E.get_value().get_string();
		DEBUG("gconf got filename=" << filename << "\n");
//		list = Shopper::List::from_file(filename);
//		DEBUG("gconf loaded file\n");		
  }
/*  if (!list)
  { // No gconf key - use the fallback....
	  list = new Shopper::List(
		  Glib::ustring(
			  "<list name='Supermarket' state='0'>"
			  "<category name='Aisle 1 - Dairy'>"
			  "<item desc='Cheese' note='note, no smelly cheese' wanted='1' bought='0'/>"
			  "</category>"
			  "</list>"
			  ));
  }
*/
  
  // Create Window and set it to Program
  MainWindow window(filename);

  Hildon::Program::get_instance()->add_window(window);

  // Begin the main application
  kit.run(window);
	
  osso_deinitialize(osso_context);

  return 0;

}



