/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */
#include "config.h"
#include <iostream>
#include <sstream>
#include <fstream>
#include <glib.h>
#include <list>
#include <set>
#include <algorithm>
#include <signal.h>
#include "shopperList.h"

//#define DEBUG(...) cerr << __VA_ARGS__
#define DEBUG(...)

using namespace std;

namespace Shopper
{
// watchable
	// Empty constructor/destructor?
	watchable::watchable()
	{}
	
	watchable::~watchable()
	{}
	
	// actions emitted by signal

	signal_changed_t watchable::watch()       /* register for a changed signal */
	{
		DEBUG("Being watched\n");
		return signal_changed;
	}
	void watchable::pause()                   /* pause emitting change signals */
	{}
	void watchable::resume()                  /* resume emitting change signals */
	{}
	Glib::ustring watchable::action_name(List_action a)  // for debugging
	{
		return "\n\n\nFIXME: action_name\n\n\n";
	}
////////////////////////////////////////////////////////////////
//  Category
	int Category::id_master = 0;
	Category::Category(sparseCategory dummy) // SPARSE constructor for ListParser
	{}
	Category::Category() :
		id (Category::id_master++)
	{
	}
	Category::Category(const Category &src)
	{
		DEBUG("\n\n\nCategory copy SCREAMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMM\n\n\n");
		exit(1);
	}
	Category::Category(Glib::ustring name) :
		name(name),
		id (Category::id_master++)
	{}
	Category::~Category()
	{
		emit_change(CATEGORY_DELETED);
	}

	void Category::dbg() {
		DEBUG(
			"\nCat  name : " << name <<
			"\n       id : " << id <<
			"\n");
	}

    // Operator
	bool operator< (const Category &a, const Category &b)
	{
		return (a.id < b.id);
	}
	bool cat_cmp (const Category *a, const Category *b)
	{
		return (a->id < b->id);
	}
    // cmp (user defined)
//	bool user_cmp (const Category &a, const Category &b)
//	{
//		return (a.pos < b.pos);
//	}

	// accessors
	Glib::ustring Category::print_id()
	{
		return "id";
	}
	int Category::get_id() {	return id; }
	void Category::set_id(int i)
	{
		id = i;
		emit_change(CATEGORY_CHANGED);
	}
	void Category::set_name(Glib::ustring n)
	{
		name = n;
		emit_change(CATEGORY_CHANGED);
	}
	Glib::ustring Category::get_name()  { return name; }

	void Category::emit_change(Shopper::List_action a)
	{
		DEBUG("Category " << name << " emit_change \n");
		signal_changed.emit(a, NULL, this, NULL);
	}

////////////////////////////////////////////////////////////////
//  Item
	int Item::id_master = 0;
	
	Item::Item(sparseItem dummy) : // SPARSE constructor for ListParser
		id (Item::id_master++),
		category (NULL)
	{}
	Item::Item() :
		id (Item::id_master++),
		category (NULL)
	{
	}
	Item::Item(Category &c, Glib::ustring d, Glib::ustring n, gboolean w, gboolean b) :
		id (Item::id_master++),
		category (&c),
		desc (d),
		note (n),
		wanted (w),
		bought (b)
	{
		DEBUG("Created an item from parts\n");
	}
	
	Item::~Item()
	{
		emit_change(ITEM_DELETED);
	}

	void Item::dbg() {
		DEBUG(
			"\nItem desc : " << desc <<
			"\n       id : " << id <<
			"\n     note : " << note <<
			"\n   bought : " << (bought ? "true":"false") <<
			"\n   wanted : " << (wanted ? "true":"false") <<
			"\n category : " << category->get_name() <<
			"\n");
	}
	// accessors
	Glib::ustring Item::print_id(){return "id";}
	int Item::get_id(){return id;}
	void Item::set_id(int i){id=i;}
	void Item::set_desc(Glib::ustring d){
		desc = d;
		emit_change(ITEM_CHANGED);
	}
	Glib::ustring Item::get_desc(){return desc;}

	void Item::set_note(Glib::ustring n)
	{
		note = n;
		emit_change(ITEM_CHANGED);
	}
	Glib::ustring Item::get_note(){return note;}
	void Item::set_wanted(gboolean w)
	{
		if (wanted !=w) {
			DEBUG("Wanted\n");
			wanted =w;
			emit_change(ITEM_CHANGED);
		}
	}
	gboolean Item::get_wanted(){return wanted;}
	void Item::set_bought(gboolean b)
	{
		if (bought != b) {
			DEBUG("Bought\n");
			bought = b;
			emit_change(ITEM_CHANGED);
		}
	}
	gboolean Item::get_bought(){return bought;}
	void Item::set_category(Category *c){
		category = c;
		emit_change(ITEM_CHANGED);
	}
	Category* Item::get_category(){return category;}
	
	void Item::emit_change(Shopper::List_action a)
	{
		DEBUG("Item " << desc << " emit_change \n");
		signal_changed.emit(a, NULL, NULL, this);
	}

////////////////////////////////////////////////////////////////							
// List

	List::List() :
		state(MAKING_LIST),
		name("Sainsburys"),
		active_category(NULL)
	{
		// No items, no categories
		DEBUG("\n\n\nMAKING NEW LIST\n\n\n");
	}
	List::List(Glib::ustring dump)
	{
		DEBUG("\n\n\nMAKING NEW LIST FROM STRING\n\n\n");
		ListParser parse(this);
		parse.from_string(dump);
	}   // initiate from a dump
	List::~List()
	{
		for(list<Category*>::iterator catI = categories.begin();
			catI != categories.end(); ++catI) {
			delete *catI;
		}
		for(list<Item*>::iterator itI = items.begin();
			itI != items.end(); ++itI) {
			delete *itI;
		}
		
	}

	List* List::from_file(Glib::ustring filename)
	{
		// open file
		std::ifstream file(filename.c_str());
		if (!file) {
			return NULL;
		}
		// Read xml as a stream
		std::stringstream xml;
		xml << file.rdbuf();
		
		// Create new list and return it
		return new Shopper::List(xml.str()); // FIXME: may not succeed.
	}

	void List::to_file(Glib::ustring filename)
	{
		std::ofstream outf(filename.c_str());
		if (!outf) {
			DEBUG("Couldn't open " << filename << " for writing\n");
			exit(1);
		}
		outf << *this ;
	}
		
	List::operator Glib::ustring() {return Glib::ustring("XML here");}    // a ustring representation of a List (XML)

    // an ostream representation of a List (XML)
	std::ostream& operator <<(std::ostream &out, const List &l)
	{
		DEBUG("in << operator\n");
		XMLWriter w(&l);
		DEBUG(w.write(cerr));
		DEBUG("written XML\n");
		return w.write(out);
	}
	
	void List::add(Item &it)
	{
		if (categories.empty()) {
			DEBUG("Adding an item but there are no categories\n");
			exit(1); // FIXME - is this a no-op or an exception or...
		}
		DEBUG("Add item " << it.get_desc() << "\n");
		items.push_back(&it);
		emit_change(ITEM_LIST_CHANGED);
	}
	void List::rm(Item &it)	   
	{
		items.remove(&it);
//		it->emit_change(ITEM_DELETED);
		delete &it;
		emit_change(ITEM_LIST_CHANGED);
	}

	void List::add(Category &cat)
	{
		DEBUG("Add category " << cat.get_name());
		if (categories.empty())
		{
			DEBUG(" made active\n");
			categories.push_back(&cat);
			make_category_active(cat);
		} else {
			list<Category*>::iterator low = lower_bound(categories.begin(),categories.end(), &cat, &cat_cmp);
			categories.insert(low, &cat);
			resequence();
		}
		DEBUG("\n");
		emit_change(CATEGORY_LIST_CHANGED);
	}
	void List::rm(Category &cat)
	{
		// FIXME : what if there are items in it?
		categories.remove(&cat);
		resequence();
		delete &cat;
		emit_change(CATEGORY_LIST_CHANGED);
	}

	// accessors
	Glib::ustring List::get_name(){return name;}
	void List::set_name(Glib::ustring)
	{
		emit_change(LIST_CHANGED);
	}
	sState List::get_state(){
		return state;
	}
	void List::set_state(sState s)
	{
		if (state != s){
			state = s;
			emit_change(LIST_STATE_CHANGED);
		}
	}

	// Category filtering
	Category* List::get_active_category()
	{
		return(active_category);
	}
	gboolean List::is_category_active(Category &c)  // empty set = all active
	{
		return (active_category == NULL || active_category == &c);
// If using a set for multiple active categories
//		return (active_categories.empty() || ( active_categories.find(c) != active_categories.end() ));;
	}
	void List::cycle_active_category(bool f)
	{
		List::pCategoryIter  begin  = categories.begin();
		List::pCategoryIter  end    = categories.end();
		if (active_category == NULL) { // use first or last element as 'next'
			active_category = f ? *(begin) : *(categories.rbegin());
		} else {
			List::pCategoryIter c =
				find(begin, end, active_category);
			if (c == end) {
				DEBUG("Not found - shouldn't happen\n");
				exit(1);
			}
			if (f) {
				active_category = (++c == end)?NULL:*c;
					} else {
				active_category = (c == begin)?NULL:*--c;
			}
		}
	}

	void List::make_next_category_active()
	{
		cycle_active_category(true);
		emit_change(LIST_ACTIVE_CAT_CHANGED);
	}
	void List::make_prev_category_active()
	{
		cycle_active_category(false);
		emit_change(LIST_ACTIVE_CAT_CHANGED);
	}
	void List::make_category_active(Category &c)
	{
		if (active_category != &c) {
			active_category = &c;
// If using a set for multiple active categories
//		active_categories.insert(c);
			emit_change(LIST_ACTIVE_CAT_CHANGED);
		}
	}
	void List::make_category_inactive(Category &c)
	{
		if (active_category != NULL) {
			active_category = NULL;
// If using a set for multiple active categories
//		active_categories.erase(c);
			emit_change(LIST_ACTIVE_CAT_CHANGED);
		}
	}
	void List::no_category_active()
	{
		if (active_category != NULL) {
			active_category = NULL;
			emit_change(LIST_ACTIVE_CAT_CHANGED);
		}
	}

	void List::resequence()
	{
		int id = 0;
		for(list<Category*>::iterator catI = categories.begin();
			catI != categories.end(); ++catI) {
			(*catI)->set_id(id++);
		}
	}
	void List::reorder()
	{
		categories.sort(&cat_cmp);
	}

	void List::swap_categories(Category *a, Category *b)
	{
		int a_id = a->get_id();
		DEBUG("a id  is : " << a->get_id()
			 << " b id  is : " << b->get_id()
			  << "\n");
		a->set_id(b->get_id());
		DEBUG("Now set b\n");
		b->set_id(a_id);
		DEBUG("a id  is : " << a->get_id()
			 << " b id  is : " << b->get_id()
			  << "\n");
		reorder();
		DEBUG("Reorder done\n");
	}
	void List::swap_categories(int a, int b)
	{
		/*Category *c1 = cat_by_id[a];
		Category *c2 = cat_by_id[b];
		c1->set_id(b);
		c2->set_id(a);
		cat_by_id[a] = c2;
		cat_by_id[b] = c1;
		reorder();*/
	}
	
	Glib::ustring List::modeText()
	{
		return modeText(state);
	}
	Glib::ustring List::modeText(sState s)
	{
		switch (s) {
		case WHATS_LEFT :
			return(Glib::ustring("What's Left"));
		case OUT_SHOPPING:
			return(Glib::ustring("Full List"));
		case MAKING_LIST:
			return(Glib::ustring("Making List"));
		default:
			return(Glib::ustring("BAD State"));
		}
	}
	List::pItemIter List::itemsI() { return items.begin(); }
	List::pCategoryIter List::categoriesI(){ return categories.begin(); }
	List::pItemIter List::itemsEnd() { return items.end(); }
	List::pCategoryIter List::categoriesEnd(){ return categories.end(); }

	void List::boughtNothing(){}  // marks all items as not bought
	void List::newList(){}        // marks all items as not wanted  

	void List::emit_change(Shopper::List_action a)
	{
		signal_changed.emit(a, this, NULL, NULL);
	}
}
