/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */
#include "shopper.h"           // automake, i8n, gettext
#include <gtkmm.h>
#include <hildon-fmmm.h>
#include <glibmm.h>
#include "CChooser.h"

//#define DEBUG(...) cerr << __VA_ARGS__
#define DEBUG(...)

namespace Shopper
{
	CChooser::CChooser() :
		Gtk::MenuToolButton()
	{}
	
	void CChooser::Init(List& l)
	{
		mylist = &l;
		// Make sure we're watchine the List for the active category to change
		mylist->watch()
			.connect(sigc::mem_fun(*this, &Shopper::CChooser::ChangeCategory));
		// and watch sure we're watchine the Category in case the
		// name of the active category changes
		Shopper::Category *c = mylist->get_active_category();
		if (c)
			cat_monitor = c->watch()
				.connect( sigc::mem_fun(*this, &Shopper::CChooser::ChangeCategory));
		set_label(c==NULL?"Everything":c->get_name());
		set_menu();
	}
	
	void CChooser::ChangeCategory(Shopper::List_action action,
							 Shopper::List *l,
							 Shopper::Category *c,
							 Shopper::Item *i)
	{
		if (action == Shopper::CATEGORY_CHANGED) {
			// The active category name changed
			set_label(c==NULL?"Everything":c->get_name());
		} else if (action == Shopper::LIST_ACTIVE_CAT_CHANGED) {
			// disconnect old monitor
			cat_monitor.disconnect();
			// and connect a new one
			Shopper::Category *c = mylist->get_active_category();
			if (c)
				cat_monitor = c->watch()
					.connect( sigc::mem_fun(*this, &Shopper::CChooser::ChangeCategory) );
			// Make sure the display is right.
			set_label(c==NULL?"Everything":c->get_name());
		}	
	}

	void CChooser::on_show_menu()
	{
		set_menu();
	}
	//categoryChooser.signal_show_menu().connect(
	//	sigc::mem_fun(*this, &MainWindow::set_categoryChooserMenu) );
	

	void CChooser::set_menu()
	{
		delete get_menu();

		Gtk::MenuItem *mi;
		Gtk::Menu *m = new Gtk::Menu();
		// it's OK to have references to the list in the actions here as
		// this menu is recreated when a new list is loaded.
		for(Shopper::List::pCategoryIter catI = mylist->categoriesI();
			catI != mylist->categoriesEnd(); ++catI)
		{
			DEBUG("Adding " << (*catI)->get_name() << " to menu\n ");
			mi = manage(new Gtk::MenuItem((*catI)->get_name()));
			mi->signal_activate().connect(
				sigc::bind( sigc::mem_fun(*mylist, &Shopper::List::make_category_active), sigc::ref(**catI)));
			m->append(*mi);
		}
		mi = manage(new Gtk::MenuItem("Everything"));
		
		mi->signal_activate().connect(
			sigc::mem_fun(*mylist, &Shopper::List::no_category_active));
		m->append(*mi);
		
		m->show_all();
		Gtk::MenuToolButton::set_menu(*m);
	}
}
