/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */
#include "shopper.h"           // automake, i8n, gettext
#include <gtkmm.h>
#include <hildon-fmmm.h>
#include "CategoryDialog.h"

#include "ItemDialog.h"

//#define DEBUG(...) cerr << __VA_ARGS__
#define DEBUG(...)

namespace Shopper
{

	ItemDialog::CatModel::CatModel() { add(col_p); add(col_name); }

////////////////////////////////////////////////////////////////
// Public
	ItemEdit::ItemEdit(Shopper::List* l, Shopper::Item* it) :
		ItemDialog(l, it)
	{}
	ItemAdd::ItemAdd(Shopper::List* l) :
		ItemDialog(l, NULL)
	{}

	int ItemDialog::run()
	{
		int response = Gtk::Dialog::run();
		if (response == Gtk::RESPONSE_ACCEPT) {
			if (myitem) {
				DEBUG("EDIT\n");
				myitem->set_desc(desc_e.get_text());
				myitem->set_category(get_category());
				DEBUG("\nEDITED:\n");
				myitem->dbg();
			} else {
				DEBUG("ADD\n");
				myitem = new Shopper::Item(*get_category(),
										   desc_e.get_text(), "", true, false);
				DEBUG("\nADDED:\n");
				myitem->dbg();
				mylist->add(*myitem);
			}
		}
		return response; // just so they know what happened.
	}

////////////////////////////////////////////////////////////////
// Private
	ItemDialog::ItemDialog(Shopper::List* list, Shopper::Item* it) :
		Gtk::Dialog("Edit Item", true, true),
		mylist(list),
		myitem(it)
	{
		// There's an OK or ADD button already.
		if (myitem) {
			add_button(Gtk::Stock::OK, Gtk::RESPONSE_ACCEPT);
		} else {
			set_title("Add Item");
			add_button(Gtk::Stock::ADD, Gtk::RESPONSE_ACCEPT);
		}
		add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);

		refTreeModel = Gtk::ListStore::create(Columns);
		Cat_combo.set_model(refTreeModel);
		Cat_combo.pack_start(Columns.col_name);
	
		Gtk::VBox *v = 	get_vbox();
		Gtk::HBox *h;
		Gtk::Label *l;
		Gtk::Button *b;
		Gtk::Image *b_i;
		h = manage(new Gtk::HBox());
		l = manage(new Gtk::Label(_("Item desc: ")));
		desc_e.set_width_chars(15);
		v->add(*h);
		h->add(*l);
		h->add(desc_e);
		if (myitem) desc_e.set_text(myitem->get_desc());
		h = manage(new Gtk::HBox());
		l = manage(new Gtk::Label(_("Category : ")));
		// A + button
		b_i = manage(new Gtk::Image(Gtk::Stock::ADD, Gtk::ICON_SIZE_BUTTON));
		b = manage(new Gtk::Button());
		b->signal_clicked().connect(sigc::mem_fun(*this, &ItemDialog::on_action_add_category) );
		b->set_image(*b_i);
		
		v->add(*h);
		h->add(*l);
		h->add(Cat_combo);
		h->add(*b);
		Cat_combo.set_active(update_combo());  // show the right category
		show_all();
	}

	int ItemDialog::update_combo() // returns the index of the active category
	{
		refTreeModel->clear();  // Start from scratch...

		Gtk::TreeModel::Row row;		
		int cat_index = 0; // default: make sure the first category is selected - no nulls.
		int c=0; // C++ won't let me declare it in for(;;) - d'oh.
		for(Shopper::List::pCategoryIter catI = mylist->categoriesI();
			catI != mylist->categoriesEnd(); ++catI,c++)
		{
//			if (myitem && myitem->get_category() == *catI ) {
			if (myitem){
				DEBUG("Editing - looking for category : is " << myitem->get_category()->get_name() << " == "<< (*catI)->get_name() << " index " << c <<"\n");
				if (myitem->get_category() == *catI ) {
					cat_index = c; // if we have an item
					DEBUG("Found category for edited item: " << c << "\n");
				}
			}
			row = *(refTreeModel->append());
			row[Columns.col_p] = *catI;
			row[Columns.col_name] = (*catI)->get_name();
		}
		return cat_index;
	}
	
	void ItemDialog::on_action_add_category()
	{
		std::cout << "A cat may be added." << std::endl;
		
		Shopper::CategoryAdd catD(mylist);
		if (catD.run() == Gtk::RESPONSE_ACCEPT) {
			DEBUG("\nADDED:\n");
		}
		Cat_combo.set_active(update_combo());  // show the active category FIXME: show the newly added one?
	}

	Shopper::Category* ItemDialog::get_category()
	{
		Gtk::TreeModel::iterator iter = Cat_combo.get_active();
		if(iter)
		{
			Gtk::TreeModel::Row row = *iter;
			Shopper::Category *c = row[Columns.col_p];
			DEBUG("Category : " << c->get_name() << " combo'ed\n");
			return(row[Columns.col_p]);
		} else {
			DEBUG("Error - no combo iterator\n");
			exit(1);
		}
	}
}
