/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <gtkmm.h>
#include <hildonmm.h>
#ifdef XRANDR
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/extensions/Xrandr.h>
#endif
#include "shopperList.h"
#include "shopperView.h"
#include "CChooser.h"


class MainWindow : public Hildon::Window
{
public:
	MainWindow(Shopper::List &l);
	MainWindow(Glib::ustring file);
	virtual ~MainWindow();

protected:
	void init_MainWindow(Shopper::List *l);
	// Main list that we're acting for
	Shopper::List *mylist;

	// types
	typedef enum _clear_list {CLEAR_WANTED,CLEAR_BOUGHT} clear_list_mode;

	//Signal handlers:
	void on_action_add_item();
	void on_action_add_category();
	void on_action_edit_category();
	void on_action_next();
	void on_action_prev();
	void on_action_clear_list(clear_list_mode m);
	void on_action_set_state(Shopper::sState s);
	void on_action_file_new();
	void on_action_file_open();
	void on_action_file_save();
	void on_action_file_saveas();
	void on_action_file_quit();
	bool on_signal_window_closed(GdkEventAny*);
	void on_action_fullscreen();
	void on_action_rotate();

	// Gui layout
	void create_list_view(Shopper::List*);

	// Gconf helper
	void save_filename(Glib::ustring name);

	// Member UI helpers and components
	Glib::RefPtr<Gtk::UIManager>   m_refUIManager;
	Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;
	Shopper::CChooser              categoryChooser;
	Shopper::ListView             *lv;
	Glib::ustring                  filename;
	
	
	/* Fullscreen mode is on (TRUE) or off (FALSE) */
    gboolean full_screen;
	bool     has_rotation;
	Rotation rotation;  // X type
};

#endif //MAINWINDOW_H
