/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef __SHOPPERVIEW_H__
#define __SHOPPERVIEW_H__

// #include <glib.h>
// #include <glib-object.h>
#include <map>
#include <gtkmm/box.h>

#include "shopperList.h"

using namespace std;

namespace Shopper
{
	class ItemView: public Gtk::HBox
	{
	public:
		ItemView(Shopper::Item &it, Shopper::List &l);
		~ItemView();
	
		void show();
		void update();
	
	private:
		Gtk::CheckButton tick;       /* display either the 'bought/not
									  * bought' or the 'wanted/not wanted'
									  * state depending on the current
									  * mode.  This also includes the
									  * label
									  */
		Gtk::HBox        outer_b;    // Capture events for labels
		Gtk::EventBox    desc_b;     // Capture events for labels
		Gtk::EventBox    note_b;
		Gtk::Label       desc;       /* This displays the item description */
		Gtk::Label       note;       /* This displays the notes. When
									  * clicked on it transforms into a
									  * textEntry */
		Gtk::Entry      *note_edit;  /* For editing the notes. Don't
									  * create this until needed */
		Item            *myitem;     /* the item being displayed */
		List            *mylist;     /* the parent list */
		
		sigc::connection watch_item;  // Keep the connections to allow block()
		sigc::connection watch_list;

		// signal handlers
		bool desc_clicked(GdkEventButton* ev=NULL);
		void tick_clicked();
		void edit_item();
		void delete_item();
		bool note_show_label(GdkEventButton* ev); // Finish on lost focus
		bool note_show_label(GdkEventFocus* ev); // Finish on lost focus
		void note_show_label();                  // Finish on activate
		bool note_show_edit(GdkEventButton* ev=NULL);
//		void note_finished();

		// data monitors
		void item_changed(List_action action, List *l, Category *c, Item *i);
		void list_changed(List_action action, List *l, Category *c, Item *i);
	};

// This class displays and manages a list.

// possibly use scrolledwindow or an viewport for finger scrolling
	class ListView: public Gtk::VBox
	{
	public:
		ListView(Shopper::List &l);
		~ListView();

		void filter_by(Shopper::Category &c);  // Only view certain categories
	private:
		Shopper::List        *mylist;
		std::map<const int, ItemView*>  shownItemVs;       // Items currently being
		                                  // displayed (may be hidden
		                                  // in what's left mode)
		sigc::connection watch_item;    
		sigc::connection watch_list;

		void ListView::list_changed(List_action action, List *l, Category *c, Item *i);

		void ListView::update();

	};
}

#endif /* __SHOPPERVIEW_H__ */
