/* Shopper
 * Copyright (C) 2008 David Greaves
 *
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */
#include "shopper.h"           // automake, i8n, gettext
#include <iostream>
#include <sstream>
#include <string>
#include <stdexcept>
#include <glib.h>
#include <list>
#include <set>
#include <signal.h>
#include "shopperList.h"

//#define DEBUG(...) cerr << __VA_ARGS__
#define DEBUG(...)

using namespace std;

namespace Shopper
{
	void abort (Glib::ustring msg){
		DEBUG(msg);
		exit(1); // FIXME
	}

	ListParser::ListParser(List *list):
		l(list)
	{
	}
	
	void ListParser::start_list_el(Glib::Markup::ParseContext& context, const AttributeMap& attr)
	{
		// zero the id counters
		Category::id_master = 0;
		Item::id_master = 0;
		Glib::ustring tmp("name");
		for (ListParser::AttributeMap::const_iterator at = attr.begin(); at != attr.end(); at++) {
			if (at->first.lowercase() == "name") {
				l->name = at->second;
			} else if (at->first.lowercase() == "state") {   // save as int? or human readable?
				std::istringstream convert(at->second);
				int st;
				if (!(convert >> st)) {
					abort("Converting state failed\n");
				}
				l->state = sState(st);
//			} else if (at->first.lowercase() == "active") {  // at end we'll need to lookup the id
//				std::istringstream convert(at->second);
//				if (!(convert >> active_cat_id)) {
//					abort("Converting active failed\n");
//				}					
			} else {
				DEBUG("<list> Unknown attribute : " << at->first << "='" <<at->second<<"'\n");
			}
		}
	}
	void ListParser::start_cat_el(Glib::Markup::ParseContext& context, const AttributeMap& attr)
	{
		if (l == NULL) {
			DEBUG("<category> Not inside <list>\n");
			exit(1); // FIXME throw?
		}
		// c = new Category(Category::SPARSE); // Not needed at the moment
		c = new Category();
		for (AttributeMap::const_iterator at = attr.begin(); at != attr.end(); at++) {
			if (at->first.lowercase() == "name") {
				c->name = at->second;
			} else if (at->first.lowercase() == "active") { // is this the category active?
				bool active;
				std::istringstream convert(at->second);
				if (!(convert >> active)) {
					abort("Converting 'active' failed\n");
				}
				l->active_category = c;
			} else {
				DEBUG("<category> Unknown attribute : " << at->first << "='" <<at->second<<"'\n");
			}
		}
		l->add(*c);
	}
	void ListParser::start_item_el(Glib::Markup::ParseContext& context, const AttributeMap& attr)
	{
		if (c == NULL) {
			DEBUG("<item> Not inside <category>\n");
			exit(1); // FIXME
		}
		// i = new Item(Item::SPARSE); // Not needed at the moment
		i = new Item();
		i->category=c; // current category
		for (AttributeMap::const_iterator at = attr.begin(); at != attr.end(); at++) {
			if (at->first.lowercase() == "desc") {
				i->desc = at->second;
//			} else if (at->first.lowercase() == "id") {
//				std::istringstream convert(at->second);
//				if (!(convert >> i->id)) {
//					abort("Converting pos failed\n");
//				}
//				Item::id_master = max(Item::id_master, i->id);
			} else if (at->first.lowercase() == "note") {
				i->note = at->second;
//			} else if (at->first.lowercase() == "cat_id") {
//				std::istringstream convert(at->second);
//				if (!(convert >> cat_id)) {
//					abort("Converting pos failed\n");
//				}
//				i->category = cat_by_id[cat_id]; // cat must be declared before item
//				if (i->category == NULL) abort("No category for item \n");
			} else if (at->first.lowercase() == "wanted") {
				i->wanted =
					(at->second.lowercase() == "true" || at->second == "1")?
					true : false;
			} else if (at->first.lowercase() == "bought") {
				i->bought =
					(at->second.lowercase() == "true" || at->second == "1")?
					true : false;
			} else {
				DEBUG("<item> Unknown attribute : " << at->first << "='" <<at->second<<"'\n");
			}
		}
		l->add(*i);
	}

	void ListParser::on_text (Glib::Markup::ParseContext& context,
										   const Glib::ustring& el)
	{
		DEBUG("Text '" << el << "'\n");
	}
	void ListParser::on_passthrough (Glib::Markup::ParseContext& context,
										   const Glib::ustring& el)
	{
		DEBUG("Pass through '" << el << "'\n");
	}
	void ListParser::on_start_element (Glib::Markup::ParseContext& context,
										   const Glib::ustring& el,
										   const AttributeMap& attr)
	{
		DEBUG("adding " << el << "\n");
		if (el.lowercase() == "list") {
			start_list_el(context, attr);
		} else if (el.lowercase() == "category") {
			start_cat_el(context, attr);
		} else if (el.lowercase() == "item") {
			start_item_el(context, attr);
		} else {
			throw ;
		}
	}
	void ListParser::on_end_element (Glib::Markup::ParseContext& context,
								 const Glib::ustring& el)
	{
		DEBUG("done " << el << "\n");
		if (el.lowercase() == "list") {
//			if (cat_by_id[active_cat_id]) { 
//				l->active_category = cat_by_id[active_cat_id];
//				DEBUG("Set default category to " << l->active_category->name << "\n");
//			} else {
//				DEBUG("Default category " << active_cat_id << " not found in map\n");
//			}
			l->resequence(); // Ensure the id's are sensible FIXME : not needed?
			l = NULL; // No current list
		} else if (el.lowercase() == "category") {
			// add the created cat to list
			c->dbg();
			c=NULL; // No current category
		} else if (el.lowercase() == "item") {
			// add the created item to list
			i->dbg();
			DEBUG("Assigned " << i->desc << " to category " << i->category->name << "\n");
			i = NULL; // No current item
		} else {
			throw ;
		}
	}
	void ListParser::on_error (Glib::Markup::ParseContext& context, const Glib::MarkupError& error)
	{
		DEBUG("Markup error\n");
	}

	void ListParser::from_string(Glib::ustring xml)
	{
		Glib::Markup::ParseContext context (*this);
		try
		{
			DEBUG("Parsing " << xml << "\n");
			context.parse(xml);
			context.end_parse();
			DEBUG("Parsing DONE\n");
		} catch (Glib::MarkupError e) {
			DEBUG("Parsing ERROR \n");
		}
		DEBUG("Parsing EXIT\n");
	}

////////////////////////////////////////////////////////////////
	XMLWriter::XMLWriter(const Shopper::List *list) :
		l(list)
	{}
		

	ostream& XMLWriter::write(ostream &out)
	{
		DEBUG("in write\n");
		out << "<list name='"<< l->name 
			<< "' state='" << l->state
			<< "'>\n";
		for (List::pCategoryIter c = l->categories.begin(); c != l->categories.end(); c++) {
			out << "  <category"
				<< ((l->active_category == *c) ? " active='1' " : "")
				<< " name='" << (*c)->name 
				<< "'>\n";
			for (List::pItemIter i = l->items.begin(); i != l->items.end(); i++) {
				if ((*i)->category != (*c)) {continue;} // only output items in this category
				out << "  <item"
					<< " wanted='" << (*i)->wanted
					<< "' bought='" << (*i)->bought
					<< "' desc='" << (*i)->desc // FIXME: Escape quotes etc
					<< "' note='" << (*i)->note // FIXME: Escape quotes etc
					<< "'/>\n";
			}
			out << "  </category>\n";
		}
		out << "</list>\n";
		DEBUG("done write\n");
		return (out);
	}
}
